from .base import Base, Condition, DayTime, Lang, MoonCode, MoonText, PhenomCondition, PrecipitationType, Season, WindDir
from .fact import Fact
from .info import Info, TzInfo
from .request import Request, RequestForecast
from .response import Response, ResponseForecast



import datetime
from typing import List, Optional

from .base import Base, Condition, DayTime, MoonCode, MoonText, PrecipitationType, WindDir


class ForecastParts(Base):
    # Nighttime forecast
    night: Optional['Forecast']
    # Morning forecast
    morning: Optional['Forecast']
    # Afternoon forecast
    day: Optional['Forecast']
    # Evening forecast
    evening: Optional['Forecast']
    # 12-hour forecast for the day
    day_short: Optional['Forecast']
    # Nighttime forecast
    night_short: Optional['Forecast']


class Forecast(Base):
    # Date of the forecast, in the format YYYY-MM-DD
    date: Optional[datetime.date]
    # The date of the forecast in Unix time
    date_ts: Optional[int]
    # Week number
    week: Optional[int]
    # Time of the sunrise in local time (may be omitted for polar regions)
    sunrise: Optional[str]
    # Time of the sunset in local time (may be omitted for polar regions)
    sunset: Optional[str]
    # Code of the lunar phase.
    moon_code: Optional[MoonCode]
    # Text code for the lunar phase.
    moon_text: Optional[MoonText]
    # Forecasts by time of day and 12-hour forecasts. 
    # Contains fields that differ by type of forecast:  
    #     Nighttime forecast
    #     Morning forecast
    #     Afternoon forecast
    #     Evening forecast
    #     12-hour forecast for the day
    #     Nighttime forecast
    # All weather forecasts for a certain time of day have the same set of fields
    # All 12-hour forecasts have the same set of fields
    # Note: For the last day returned in the forecast, some of the parts might be missing
    parts: Optional[ForecastParts]
    # Object with the weather forecast for the night
    # The beginning of the nighttime period corresponds to the beginning of the 24-hour period
    # To specify the upcoming night temperatures, use the object for the nighttime forecast for the next day
    night: Optional[ForecastParts]
    # Minimum temperature for the time of day (°C)
    temp_min: Optional[float]
    # Maximum temperature for the time of day (°C)
    temp_max: Optional[float]
    # Average temperature for the time of day (°C)
    temp_avg: Optional[float]
    # What the temperature feels like (°C)
    feels_like: Optional[float]
    # The code of the weather icon
    icon: Optional[str]
    # The code for the weather description
    condition: Optional[Condition]
    # Light or dark time of the day
    daytime: Optional[DayTime]
    # Indicates that the time of day specified in the daytime field is polar
    polar: Optional[bool]
    # Wind speed (meters per second)
    wind_speed: Optional[float]
    # Speed of wind gusts (meters per second)
    wind_gust: Optional[float]
    # Wind direction
    wind_dir: Optional[WindDir]
    # Atmospheric pressure (mm Hg)
    pressure_mm: Optional[int]
    # Atmospheric pressure (hPa)
    pressure_pa: Optional[int]
    # Humidity (percent)
    humidity: Optional[float]
    # Predicted amount of precipitation (mm)
    prec_mm: Optional[int]
    # Predicted duration of precipitation (minutes)
    prec_period: Optional[int]
    # Type of precipitation
    prec_type: Optional[PrecipitationType]
    # Intensity of precipitation
    # Possible values:
    #    0.00 — No precipitation
    #    0.25 — Light rain or snow
    #    0.50 — Rain or snow
    #    0.75 — Heavy rain or snowfall
    #    1.00 — Heavy downpour or snowstorm
    prec_strength: Optional[float]
    # Cloud cover
    # Possible values:
    #    0.00 — Clear
    #    0.25 — Partly cloudy
    #    0.50 — Cloudy
    #    0.75 — Cloudy
    #    1.00 — Overcast
    cloudness: Optional[float]
    # Object with a 12-hour forecast for the day
    day_short: Optional[ForecastParts]
    # Highest daytime or lowest nighttime temperature (°C)
    temp: Optional[float]
    # Object for the hourly forecast
    # Consists of 24 parts (hours) for the first 2-3 days, then an empty string is returned
    # Each part contains the following fields:
    #    hour hour_ts temp feels_like icon condition wind_speed wind_gust wind_dir pressure_mm pressure_pa humidity prec_mm prec_period prec_type prec_strength is_thunder cloudness
    hours: Optional[List[dict]]
    # The hour the forecast is for (0-23) using the local time
    hour: Optional[str]
    # The time of the forecast in Unix time
    hour_ts: Optional[float]

    @property
    def icon_url(self) -> Optional[str]:
        return self.icon and f'https://yastatic.net/weather/i/icons/blueye/color/svg/{self.icon}.svg'


ForecastParts.update_forward_refs()



from typing import Optional

from pydantic import PositiveInt

from .base import Base, Lang


class Request(Base):
    def prepare(self) -> dict:
        d = self.dict(exclude_none=True)
        for k, v in d.items():
            if isinstance(v, (bool, float)):
                d[k] = str(v)
        return d


class RequestForecast(Request):
    # The latitude in degrees
    lat: float
    # The longitude in degrees
    lon: float
    # The combination of language and country that weather information will be returned for
    lang: Optional[Lang]
    # The number of days in the forecast, including the current day
    limit: Optional[PositiveInt]
    # For each day, the response will contain the hourly weather forecast
    hours: Optional[bool]
    # Extra information about precipitation
    extra: Optional[bool]



"""Yandex Weather API with asyncio support and typings"""

from . import cities
from .cities import Afghanistan, Albania, Algeria, AmericanSamoa, Andorra, Angola, AntiguaAndBarbuda, Argentina, Armenia, Aruba, Australia, \
    Austria, Azerbaijan, Bahamas, Bahrain, Bangladesh, Barbados, Belarus, Belgium, Belize, Benin, Bhutan, Bolivia, BosniaAndHerzegovina, \
    Botswana, Brazil, Brunei, Bulgaria, BurkinaFaso, Burma, Burundi, CaboVerde, Cambodia, Cameroon, Canada, CentralAfricanRepublic, Chad, \
    Chile, China, Colombia, Comoros, CongoBrazzaville, CongoKinshasa, CostaRica, CoteDIvoire, Croatia, Cuba, Curacao, Cyprus, Czechia, \
    Denmark, Djibouti, Dominica, DominicanRepublic, Ecuador, Egypt, ElSalvador, EquatorialGuinea, Eritrea, Estonia, Ethiopia, Fiji, Finland, \
    France, Gabon, Gambia, Georgia, Germany, Ghana, Greece, Grenada, Guam, Guatemala, Guinea, GuineaBissau, Guyana, Haiti, Honduras, \
    HongKong, Hungary, Iceland, India, Indonesia, Iran, Iraq, Ireland, Israel, Italy, Jamaica, Japan, Jordan, Kazakhstan, Kenya, Kiribati, \
    KoreaNorth, KoreaSouth, Kosovo, Kuwait, Kyrgyzstan, Laos, Latvia, Lebanon, Lesotho, Liberia, Libya, Liechtenstein, Lithuania, \
    Luxembourg, Macau, Macedonia, Madagascar, Malawi, Malaysia, Maldives, Mali, Malta, MarshallIslands, Mauritania, Mauritius, Mexico, \
    Micronesia, Moldova, Monaco, Mongolia, Montenegro, Morocco, Mozambique, Namibia, Nepal, Netherlands, NewZealand, Nicaragua, Niger, \
    Nigeria, NorthernMarianaIslands, Norway, Oman, Pakistan, Palau, Panama, PapuaNewGuinea, Paraguay, Peru, Philippines, Poland, Portugal, \
    Qatar, Romania, Russia, Rwanda, SaintKittsAndNevis, SaintLucia, SaintVincentAndTheGrenadines, Samoa, SanMarino, SaoTomeAndPrincipe, \
    SaudiArabia, Senegal, Serbia, Seychelles, SierraLeone, Singapore, SintMaarten, Slovakia, Slovenia, SolomonIslands, Somalia, SouthAfrica, \
    SouthSudan, Spain, SriLanka, Sudan, Suriname, Swaziland, Sweden, Switzerland, Syria, Taiwan, Tajikistan, Tanzania, Thailand, TimorLeste, \
    Togo, Tonga, TrinidadAndTobago, Tunisia, Turkey, Turkmenistan, Tuvalu, Uganda, Ukraine, UnitedArabEmirates, UnitedKingdom, UnitedStates, \
    Uruguay, Uzbekistan, Vanuatu, Venezuela, Vietnam, WestBank, Yemen, Zambia, Zimbabwe
from .main import YaWeather, YaWeatherAPIError, YaWeatherAsync, YaWeatherBase
from .models import Base, Condition, DayTime, Fact, Info, Lang, MoonCode, MoonText, PhenomCondition, PrecipitationType, Request, \
    RequestForecast, Response, ResponseForecast, Season, TzInfo, WindDir

__author__ = 'uburuntu'
__email__ = 'github@rmbk.me'

__license__ = 'MIT'
__version__ = '1.1.5'

__all__ = (
    'Base',
    'Condition',
    'DayTime',
    'Fact',
    'Info',
    'Lang',
    'MoonCode',
    'MoonText',
    'PhenomCondition',
    'PrecipitationType',
    'Request',
    'RequestForecast',
    'Response',
    'ResponseForecast',
    'Season',
    'TzInfo',
    'WindDir',
    'YaWeather',
    'YaWeatherAsync',
    'YaWeatherBase',
    'YaWeatherAPIError',
    'cities',

    # Countries
    'Afghanistan',
    'Albania',
    'Algeria',
    'AmericanSamoa',
    'Andorra',
    'Angola',
    'AntiguaAndBarbuda',
    'Argentina',
    'Armenia',
    'Aruba',
    'Australia',
    'Austria',
    'Azerbaijan',
    'Bahamas',
    'Bahrain',
    'Bangladesh',
    'Barbados',
    'Belarus',
    'Belgium',
    'Belize',
    'Benin',
    'Bhutan',
    'Bolivia',
    'BosniaAndHerzegovina',
    'Botswana',
    'Brazil',
    'Brunei',
    'Bulgaria',
    'BurkinaFaso',
    'Burma',
    'Burundi',
    'CaboVerde',
    'Cambodia',
    'Cameroon',
    'Canada',
    'CentralAfricanRepublic',
    'Chad',
    'Chile',
    'China',
    'Colombia',
    'Comoros',
    'CongoBrazzaville',
    'CongoKinshasa',
    'CostaRica',
    'CoteDIvoire',
    'Croatia',
    'Cuba',
    'Curacao',
    'Cyprus',
    'Czechia',
    'Denmark',
    'Djibouti',
    'Dominica',
    'DominicanRepublic',
    'Ecuador',
    'Egypt',
    'ElSalvador',
    'EquatorialGuinea',
    'Eritrea',
    'Estonia',
    'Ethiopia',
    'Fiji',
    'Finland',
    'France',
    'Gabon',
    'Gambia',
    'Georgia',
    'Germany',
    'Ghana',
    'Greece',
    'Grenada',
    'Guam',
    'Guatemala',
    'Guinea',
    'GuineaBissau',
    'Guyana',
    'Haiti',
    'Honduras',
    'HongKong',
    'Hungary',
    'Iceland',
    'India',
    'Indonesia',
    'Iran',
    'Iraq',
    'Ireland',
    'Israel',
    'Italy',
    'Jamaica',
    'Japan',
    'Jordan',
    'Kazakhstan',
    'Kenya',
    'Kiribati',
    'KoreaNorth',
    'KoreaSouth',
    'Kosovo',
    'Kuwait',
    'Kyrgyzstan',
    'Laos',
    'Latvia',
    'Lebanon',
    'Lesotho',
    'Liberia',
    'Libya',
    'Liechtenstein',
    'Lithuania',
    'Luxembourg',
    'Macau',
    'Macedonia',
    'Madagascar',
    'Malawi',
    'Malaysia',
    'Maldives',
    'Mali',
    'Malta',
    'MarshallIslands',
    'Mauritania',
    'Mauritius',
    'Mexico',
    'Micronesia',
    'Moldova',
    'Monaco',
    'Mongolia',
    'Montenegro',
    'Morocco',
    'Mozambique',
    'Namibia',
    'Nepal',
    'Netherlands',
    'NewZealand',
    'Nicaragua',
    'Niger',
    'Nigeria',
    'NorthernMarianaIslands',
    'Norway',
    'Oman',
    'Pakistan',
    'Palau',
    'Panama',
    'PapuaNewGuinea',
    'Paraguay',
    'Peru',
    'Philippines',
    'Poland',
    'Portugal',
    'Qatar',
    'Romania',
    'Russia',
    'Rwanda',
    'SaintKittsAndNevis',
    'SaintLucia',
    'SaintVincentAndTheGrenadines',
    'Samoa',
    'SanMarino',
    'SaoTomeAndPrincipe',
    'SaudiArabia',
    'Senegal',
    'Serbia',
    'Seychelles',
    'SierraLeone',
    'Singapore',
    'SintMaarten',
    'Slovakia',
    'Slovenia',
    'SolomonIslands',
    'Somalia',
    'SouthAfrica',
    'SouthSudan',
    'Spain',
    'SriLanka',
    'Sudan',
    'Suriname',
    'Swaziland',
    'Sweden',
    'Switzerland',
    'Syria',
    'Taiwan',
    'Tajikistan',
    'Tanzania',
    'Thailand',
    'TimorLeste',
    'Togo',
    'Tonga',
    'TrinidadAndTobago',
    'Tunisia',
    'Turkey',
    'Turkmenistan',
    'Tuvalu',
    'Uganda',
    'Ukraine',
    'UnitedArabEmirates',
    'UnitedKingdom',
    'UnitedStates',
    'Uruguay',
    'Uzbekistan',
    'Vanuatu',
    'Venezuela',
    'Vietnam',
    'WestBank',
    'Yemen',
    'Zambia',
    'Zimbabwe',
)



from enum import Enum, IntEnum, auto

from pydantic import BaseConfig, BaseModel, Extra


class Base(BaseModel):
    class Config(BaseConfig):
        extra = Extra.allow
        use_enum_values = True


class MoonCode(IntEnum):
    full_moon = 0
    waning_crescent_1 = auto()
    waning_crescent_2 = auto()
    waning_crescent_3 = auto()
    last_quarter = auto()
    waning_gibbous_1 = auto()
    waning_gibbous_2 = auto()
    waning_gibbous_3 = auto()
    new_moon = auto()
    waxing_gibbous_1 = auto()
    waxing_gibbous_2 = auto()
    waxing_gibbous_3 = auto()
    first_quarter = auto()
    waxing_crescent_1 = auto()
    waxing_crescent_2 = auto()
    waxing_crescent_3 = auto()


class MoonText(str, Enum):
    full_moon = 'moon-code-0'
    waning_crescent_1 = 'moon-code-1'
    waning_crescent_2 = 'moon-code-2'
    waning_crescent_3 = 'moon-code-3'
    last_quarter = 'moon-code-4'
    waning_gibbous_1 = 'moon-code-5'
    waning_gibbous_2 = 'moon-code-6'
    waning_gibbous_3 = 'moon-code-7'
    new_moon = 'moon-code-8'
    waxing_gibbous_1 = 'moon-code-9'
    waxing_gibbous_2 = 'moon-code-10'
    waxing_gibbous_3 = 'moon-code-11'
    first_quarter = 'moon-code-12'
    waxing_crescent_1 = 'moon-code-13'
    waxing_crescent_2 = 'moon-code-14'
    waxing_crescent_3 = 'moon-code-15'


class Condition(str, Enum):
    clear = 'clear'
    partly_cloudy = 'partly-cloudy'
    cloudy = 'cloudy'
    overcast = 'overcast'
    drizzle = 'drizzle'
    light_rain = 'light-rain'
    rain = 'rain'
    moderate_rain = 'moderate-rain'
    heavy_rain = 'heavy-rain'
    continuous_heavy_rain = 'continuous-heavy-rain'
    showers = 'showers'
    wet_snow = 'wet-snow'
    light_snow = 'light-snow'
    snow = 'snow'
    snow_showers = 'snow-showers'
    hail = 'hail'
    thunderstorm = 'thunderstorm'
    thunderstorm_with_rain = 'thunderstorm-with-rain'
    thunderstorm_with_hail = 'thunderstorm-with-hail'


class PhenomCondition(str, Enum):
    fog = 'fog'
    mist = 'mist'
    smoke = 'smoke'
    dust = 'dust'
    dust_suspension = 'dust-suspension'
    duststorm = 'duststorm'
    thunderstorm = 'thunderstorm'
    drifting_snow = 'drifting-snow'
    blowing_snow = 'blowing-snow'
    ice_pellets = 'ice-pellets'
    freezing_rain = 'freezing-rain'
    tornado = 'tornado'
    volcanic_ash = 'volcanic-ash'


class DayTime(str, Enum):
    # Light time of day
    d = 'd'
    # Dark time of day
    n = 'n'


class WindDir(str, Enum):
    # Northwest
    nw = 'nw'
    # North
    n = 'n'
    # Northeast
    ne = 'ne'
    # East
    e = 'e'
    # Southeast
    se = 'se'
    # South
    s = 's'
    # Southwest
    sw = 'sw'
    # West
    w = 'w'
    # Calm
    c = 'c'


class PrecipitationType(IntEnum):
    no_precipitation = 0
    rain = 1
    sleet = 2
    snow = 3
    hail = 4


class Season(str, Enum):
    summer = 'summer'
    autumn = 'autumn'
    winter = 'winter'
    spring = 'spring'


class Lang(str, Enum):
    # Russian for the Russia locale
    ru_RU = 'ru_RU'
    # Russian for the Ukraine locale
    ru_UA = 'ru_UA'
    # Ukrainian for the Ukraine locale
    uk_UA = 'uk_UA'
    # Belarusian for the Belarus locale
    be_BY = 'be_BY'
    # Kazakh for the Kazakhstan locale
    kk_KZ = 'kk_KZ'
    # Turkish for the Turkey locale
    tr_TR = 'tr_TR'
    # International English
    en_US = 'en_US'



from typing import Optional

from .base import Base, Condition, DayTime, PhenomCondition, PrecipitationType, Season, WindDir


class Fact(Base):
    # Temperature (°C)
    temp: float
    # What the temperature feels like (°C)
    feels_like: float
    # The water temperature (°C). This parameter is returned for localities where this information is relevant
    temp_water: Optional[float]
    # The code of the weather icon.
    icon: str
    # The code for the weather description
    condition: Condition
    # Wind speed (meters per second)
    wind_speed: float
    # Speed of wind gusts (meters per second)
    wind_gust: float
    # Wind direction
    wind_dir: WindDir
    # Atmospheric pressure (mm Hg)
    pressure_mm: int
    # Atmospheric pressure (hPa)
    pressure_pa: int
    # Humidity (percent)
    humidity: float
    # Light or dark time of the day
    daytime: DayTime
    # Indicates that the time of day specified in the daytime field is polar
    polar: bool
    # Time of year in this locality
    season: Season
    # The time when weather data was measured, in Unix time
    obs_time: int
    # Indicates a thunderstorm
    is_thunder: Optional[bool]
    # Type of precipitation
    prec_type: Optional[PrecipitationType]
    # Intensity of precipitation
    # Possible values:
    #    0.00 — No precipitation
    #    0.25 — Light rain or snow
    #    0.50 — Rain or snow
    #    0.75 — Heavy rain or snowfall
    #    1.00 — Heavy downpour or snowstorm
    prec_strength: Optional[float]
    # Cloud cover
    # Possible values:
    #    0.00 — Clear
    #    0.25 — Partly cloudy
    #    0.50 — Cloudy
    #    0.75 — Cloudy
    #    1.00 — Overcast
    cloudness: Optional[float]
    # The code for an additional weather event icon
    phenom_icon: Optional[str]
    # The code for an additional weather description
    phenom_condition: Optional[PhenomCondition]

    @property
    def icon_url(self) -> Optional[str]:
        return self.icon and f'https://yastatic.net/weather/i/icons/blueye/color/svg/{self.icon}.svg'

    @property
    def phenom_icon_url(self) -> Optional[str]:
        return self.phenom_icon and f'https://yastatic.net/weather/i/icons/blueye/color/svg/{self.phenom_icon}.svg'



import datetime
from typing import List

from .base import Base
from .fact import Fact
from .forecast import Forecast
from .info import Info


class Response(Base):
    pass


class ResponseForecast(Response):
    now: int
    now_dt: datetime.datetime
    info: Info
    fact: Fact
    forecasts: List[Forecast]



from .base import Base


class TzInfo(Base):
    # Time zone in seconds from UTC
    offset: int
    # Name of the time zone
    name: str
    # Abbreviated name of the time zone
    abbr: str
    # Daylight saving time
    dst: bool


class Info(Base):
    # The latitude (in degrees).
    lat: float
    # The longitude (in degrees)
    lon: float
    # Information about the time zone
    tzinfo: TzInfo
    # The normal pressure for the given coordinates (mm Hg)
    def_pressure_mm: int
    # The normal pressure for the given coordinates (hPa)
    def_pressure_pa: int
    # Locality page on Yandex.Weather (https://yandex.ru/pogoda/)
    url: str



import os
from importlib.machinery import SourceFileLoader

import setuptools
from pkg_resources import parse_requirements

module_name = 'yaweather'

module = SourceFileLoader(
    module_name, os.path.join(module_name, '__init__.py')
).load_module()


def read(filename):
    with open(filename, encoding='utf-8') as file:
        return file.read()


def load_requirements(filename: str) -> list:
    requirements = []
    for req in parse_requirements(read(filename)):
        extras = '[{}]'.format(','.join(req.extras)) if req.extras else ''
        requirements.append(
            '{}{}{}'.format(req.name, extras, req.specifier)
        )
    return requirements


setuptools.setup(
    name=module_name,
    version=module.__version__,
    author=module.__author__,
    author_email=module.__email__,
    license=module.__license__,
    description=module.__doc__,
    platforms='all',
    long_description=read('readme.md'),
    long_description_content_type='text/markdown',
    url='https://github.com/uburuntu/{}'.format(module_name),
    download_url='https://github.com/uburuntu/{}/archive/master.zip'.format(module_name),
    packages=setuptools.find_packages(exclude=['examples', 'tests']),
    requires_python='>=3.6',
    install_requires=load_requirements('requirements.txt'),
    extras_require={'dev': load_requirements('requirements-dev.txt')},
    keywords=['yandex', 'ya', 'weather', 'api'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: AsyncIO',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Typing :: Typed',
    ],
)



import json
from typing import Tuple

import aiohttp
import requests

from .models import Lang, Request, RequestForecast, ResponseForecast


class YaWeatherAPIError(Exception):
    pass


class YaWeatherBase:
    """
    Docs: https://tech.yandex.com/weather/doc/dg/concepts/forecast-test-docpage/
    Docs RU: https://yandex.ru/dev/weather/doc/dg/concepts/forecast-test-docpage/
    """
    api_url = 'https://api.weather.yandex.ru/v2/'

    def __init__(self,
                 api_key: str,
                 lang: Lang = None,
                 limit: int = None,
                 hours: bool = None,
                 extra: bool = None):
        """
        YaWeather initializer

        :param api_key: Token from https://developer.tech.yandex.ru/services/18/
        :param lang: The combination of language and country that weather information will be returned for, defaults to 'en_US'
        :param limit: The number of days in the forecast, including the current day
        :param hours: For each day, the response will contain the hourly weather forecast, defaults to True
        :param extra: Extra information about precipitation, defaults to False
        """
        self.api_key = api_key
        self.lang = lang or Lang.en_US
        self.limit = limit
        self.hours = hours
        self.extra = extra


class YaWeather(YaWeatherBase):
    @property
    def session(self) -> requests.Session:
        session = getattr(self, '_session', None)
        if session is None:
            session = requests.Session()
            session.headers['X-Yandex-API-Key'] = self.api_key
            setattr(self, '_session', session)
        return session

    def close(self):
        """
        Closes session
        :return: None
        """
        self.session.close()

    def __enter__(self):
        return self

    def __aexit__(self, exc_type, exc_val, exc_tb):
        return self.close()

    def _request(self, endpoint: str, request: Request) -> bytes:
        response = self.session.get(self.api_url + endpoint, params=request.prepare())
        if response.status_code != 200:
            raise YaWeatherAPIError(f'[{response.status_code}] {response.reason}')
        result = response.content
        return result

    def _forecast(
            self, coordinates: Tuple[float, float],
            lang: Lang = None, limit: int = None, hours: bool = None, extra: bool = None
    ) -> bytes:
        request = RequestForecast(
            lat=coordinates[0],
            lon=coordinates[1],
            lang=self.lang if lang is None else lang,
            limit=self.limit if limit is None else limit,
            hours=self.hours if hours is None else hours,
            extra=self.extra if extra is None else extra,
        )
        result = self._request('forecast', request)
        return result

    def forecast_raw(
            self, coordinates: Tuple[float, float],
            lang: Lang = None, limit: int = None, hours: bool = None, extra: bool = None
    ) -> dict:
        """
        Request forecast and return API response as raw dict

        :param coordinates: Tuple of two floats: longitude and latitude
        :param lang: The combination of language and country that weather information will be returned for
        :param limit: The number of days in the forecast, including the current day
        :param hours: For each day, the response will contain the hourly weather forecast
        :param extra: Extra information about precipitation
        :return: dict
        """
        result = self._forecast(coordinates, lang, limit, hours, extra)
        return json.loads(result)

    def forecast(
            self, coordinates: Tuple[float, float],
            lang: Lang = None, limit: int = None, hours: bool = None, extra: bool = None
    ) -> ResponseForecast:
        """
        Request forecast and return API response as instance of ResponseForecast

        :param coordinates: Tuple of two floats: longitude and latitude
        :param lang: The combination of language and country that weather information will be returned for
        :param limit: The number of days in the forecast, including the current day
        :param hours: For each day, the response will contain the hourly weather forecast
        :param extra: Extra information about precipitation
        :return: ResponseForecast
        """
        result = self._forecast(coordinates, lang, limit, hours, extra)
        return ResponseForecast.parse_raw(result)


class YaWeatherAsync(YaWeatherBase):
    @property
    def session(self) -> aiohttp.ClientSession:
        session = getattr(self, '_session', None)
        if session is None:
            session = aiohttp.ClientSession(headers={'X-Yandex-API-Key': self.api_key})
            setattr(self, '_session', session)
        return session

    async def close(self):
        """
        Closes session
        :return: None
        """
        await self.session.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        return await self.close()

    async def _request(self, endpoint: str, request: Request) -> bytes:
        async with self.session.get(self.api_url + endpoint, params=request.prepare()) as response:
            if response.status != 200:
                raise YaWeatherAPIError(f'[{response.status}] {response.reason}')
            result = await response.read()
            return result

    async def _forecast(
            self, coordinates: Tuple[float, float],
            lang: Lang = None, limit: int = None, hours: bool = None, extra: bool = None
    ) -> bytes:
        request = RequestForecast(
            lat=coordinates[0],
            lon=coordinates[1],
            lang=self.lang if lang is None else lang,
            limit=self.limit if limit is None else limit,
            hours=self.hours if hours is None else hours,
            extra=self.extra if extra is None else extra,
        )
        result = await self._request('forecast', request)
        return result

    async def forecast_raw(
            self, coordinates: Tuple[float, float],
            lang: Lang = None, limit: int = None, hours: bool = None, extra: bool = None
    ) -> dict:
        """
        Request forecast and return API response as raw dict

        :param coordinates: Tuple of two floats: longitude and latitude
        :param lang: The combination of language and country that weather information will be returned for
        :param limit: The number of days in the forecast, including the current day
        :param hours: For each day, the response will contain the hourly weather forecast
        :param extra: Extra information about precipitation
        :return: dict
        """
        result = await self._forecast(coordinates, lang, limit, hours, extra)
        return json.loads(result)

    async def forecast(
            self, coordinates: Tuple[float, float],
            lang: Lang = None, limit: int = None, hours: bool = None, extra: bool = None
    ) -> ResponseForecast:
        """
        Request forecast and return API response as instance of ResponseForecast

        :param coordinates: Tuple of two floats: longitude and latitude
        :param lang: The combination of language and country that weather information will be returned for
        :param limit: The number of days in the forecast, including the current day
        :param hours: For each day, the response will contain the hourly weather forecast
        :param extra: Extra information about precipitation
        :return: ResponseForecast
        """
        result = await self._forecast(coordinates, lang, limit, hours, extra)
        return ResponseForecast.parse_raw(result)
